	const express    = require('express');
	const exec       = require('child_process').exec;		

//These functions handle everything regarding the frontend of ExplorViz except adding it to the zip. Each function returns a Promise.

function prepareClientFolder(clientName, fileName){
	return new Promise((resolve,reject)=>{

		exec('cp -R ' + fileName + '/ ' + clientName + '/',(error,stdout,stderr)=>{

			if (error) {
				reject(error);
				return;
			}

			console.log(stdout);
			console.log(stderr);

			resolve(fileName);

		})	
	})


}

//remove addons from former builds
function frontendClean(clientName, fileName){
	return new Promise((resolve, reject) => {

		const path= process.cwd() + '/'+ clientName + '/' + fileName;
		exec('git checkout package.json && npm prune ', {cwd:path}, (error, stdout, stderr) => {
			
			if (error) {
				reject(error)
				return;
			}

			console.log(stdout);
			console.log(stderr);

			resolve();
		});
	})

}


function frontendPull(clientName, fileName){

	return new Promise((resolve, reject) => {

		const path= process.cwd() + '/'+ clientName + '/' + fileName;
		exec('git pull ', {cwd:path}, (error, stdout, stderr) => {
			
			if (error) {
				reject(error)
				return;
			}

			console.log(stdout);
			console.log(stderr);

			resolve();
		});
	})
} 

function frontendInstallAddons(clientName, fileName, checkedFiles, downloadFiles){
	let pluginsFinished=0;
	return new Promise((resolve, reject) => {

		checkedPlugins = checkedFiles.filter(function(file){
			return /\bexplorviz-frontend-plugin-\w*/.exec(file);
		});

		if(checkedPlugins.length !== 0 ){
			checkedPlugins.forEach((pluginName) => {

				const path= process.cwd() + '/'+ clientName + '/' + fileName;
					//--save: addon is listed under dependencies-section in package.json
					exec('ember install ' + downloadFiles[pluginName] + ' --save', {cwd:path}, (error, stdout, stderr) => {
						
						if (error) {
							reject(error);
							return;
						}

						console.log(stdout);
						console.log(stderr);

						pluginsFinished = pluginsFinished +1;

						if (checkedPlugins.length == pluginsFinished){
							resolve();
						}
					});

				})


		} else {

			//If no plugins were checked by the user, the downloader skips the installation step by resolving immediately.
			resolve();
		}

	})
}

function frontendBuild(clientName, fileName){

	return new Promise((resolve,reject)=> {
		
		const path= process.cwd() + '/'+ clientName + '/' + fileName;
		exec('ember build --environment=production', {cwd:path}, (error, stdout, stderr) => {
			
			if (error) {
				reject(error);
				return;
			}

			console.log(stdout);
			console.log(stderr);

			resolve({zipFileName:'frontend', path: clientName + '/explorviz-ui-frontend/dist/'});
		});



	})

}

module.exports = {
	prepareClientFolder,
	frontendClean,
	frontendPull,
	frontendInstallAddons,
	frontendBuild
}
