const express = require('express'); 
const JSZip = require('jszip'); 
const fs         = require('fs');

module.exports= function(app){

const downloadNames = app.get('downloadNames');
const downloadFiles = app.get('downloadFiles');
const clientFolder = app.get('clientFolder');
const backendFunctions = app.get('backendFunctions');
const frontendFunctions = app.get('frontendFunctions');
const zipFunctions = app.get('zipFunctions'); 

//This route gets the files the user checked for downloading, builds the custom build and delivers the zip file as a download.
app.get('/download', function(req, res, next){
	const checkedFiles = req.query.checkedFiles.split(',');
	let clientName;
	let clientCounter = req.app.locals.clientCounter;
	const zip = new JSZip();
	zip.file("README.txt", "text for README");
		//holds promises for backend and/or frontend for synchronizing
		const promiseArray = []
		
		//TODO handle multiple clients
		if (clientCounter > 5){
			res.render('tooManyClients');
		}else{
			clientName = 'client_' + clientCounter;
			clientFolder.makeClientFolder(clientName);
			
		}
		
		backendChecked = checkedFiles.find(function(file){
			return file === 'explorviz-ui-backend';
		});
		frontendChecked = checkedFiles.find(function(file){
			return file === 'explorviz-ui-frontend';
		});
//The user is not allowed to check plugins without having checked frontend or backend core. 
//If he has not checked at least one of those, he will be redirected to the homepage.
if(!(backendChecked || frontendChecked)){
	res.render('home', {downloadFiles: downloadNames});
}

if (backendChecked){

	const fileName = backendChecked;

	function backendAll(){
		return backendFunctions.prepareClientFolder(clientName,fileName).then(function(){
			return backendFunctions.backendClean(clientName, fileName).then(function(){
				return backendFunctions.backendPull(clientName, fileName).then(function(){
					return backendFunctions.backendInstallAddons(checkedFiles,downloadFiles,clientName).then(function(){
						return backendFunctions.backendBuild(checkedFiles, clientName, fileName)
					})
				})
			})
		}).catch(function(error){
			let response = 
			'There was an error during your backend build. ' +
			'Please contact "explorviz-developers-request@listserv.dfn.de" ' +
			'and add the following information: ' + error;
			res.send(response);

		})

	}
	promiseArray.push(backendAll());

} 



if (frontendChecked){
	const fileName = frontendChecked;
	const downloadURL = downloadFiles[frontendChecked]; 

	function frontendAll() {
		return frontendFunctions.prepareClientFolder(clientName,fileName).then(function(){
			return frontendFunctions.frontendClean(clientName,fileName).then(function(){
				return frontendFunctions.frontendPull(clientName,fileName).then(function(){
					return frontendFunctions.frontendInstallAddons(clientName,fileName, checkedFiles, downloadFiles).then(function(){
						return frontendFunctions.frontendBuild(clientName,fileName)
					})
				})
			})
		}).catch(function(error){
			let response = 
			'There was an error during your frontend build. ' +
			'Please contact "explorviz-developers-request@listserv.dfn.de" ' +
			'and add the following information: ' + error;
			res.send(response);

		})
	}
	promiseArray.push(frontendAll());

};

finalizeZip();

console.log('promiseArray:', promiseArray);
function finalizeZip(){
	return zipFunctions.streamFilesToZip(promiseArray,zip, clientName).then(function(zip){
		return createZip(zip)
	}).catch(function(error) {let response = 
		'There was an error during your build. ' +
		'Please contact "explorviz-developers-request@listserv.dfn.de" ' +
		'and add the following information: ' + error;
		res.send(response);
	})

}

function createZip(zip){
	return new Promise((resolve,reject) => {
		zip.generateNodeStream({type:'nodebuffer',streamFiles:true}).pipe(fs.createWriteStream('explorviz-builds.zip')).on('error', function(error){
			console.log('error in createZip:', error);
		}).on('finish', function () {
			console.log('explorviz-builds.zip written.');
			//client finishes everything except getting the download
			req.app.locals.clientCounter = clientCounter - 1;
			res.download('explorviz-builds.zip', function(error){
				if(error){
					let response = 
					'There was an error during your build. ' +
					'Please contact "explorviz-developers-request@listserv.dfn.de" ' +
					'and add the following information: ' + error;
					res.send(response);
				} 
			});
		}); 

	})

	
}


});

}
